/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIDOMNode;

[scriptable, uuid(5BEC321F-59AC-413a-A4AD-8A8D7C50A0D0)]
interface nsITextServicesFilter : nsISupports
{

  /**
   * Indicates whether the content node should be skipped by the iterator
   *  @param aNode - node to skip
   */
  boolean skip(in nsIDOMNode aNode);

};

