/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMHTMLElement.idl"

/**
 * The nsIDOMHTMLLinkElement interface is the interface to a [X]HTML
 * link element.
 *
 * This interface is trying to follow the DOM Level 2 HTML specification:
 * http://www.w3.org/TR/DOM-Level-2-HTML/
 *
 * with changes from the work-in-progress WHATWG HTML specification:
 * http://www.whatwg.org/specs/web-apps/current-work/
 */

[scriptable, uuid(95d6ec66-2754-45bd-a068-49ac1fb45004)]
interface nsIDOMHTMLLinkElement : nsISupports
{
           [binaryname(MozDisabled)]
           attribute boolean          disabled;
           attribute DOMString        charset;
           attribute DOMString        href;
           attribute DOMString        hreflang;
           attribute DOMString        media;
           attribute DOMString        rel;
           attribute DOMString        rev;
           attribute DOMString        target;
           attribute DOMString        type;
           attribute DOMString        crossOrigin;
};
